""" Install additional dependencies into a Kraken build environment. """


from __future__ import annotations

import hashlib
import logging
import subprocess as sp
import sys
from pathlib import Path

import localimport

logger = logging.getLogger(__name__)


class PyenvManager:
    def __init__(self, lib_directory: Path) -> None:
        """Install any missing dependencies into a new location as per the project buildscript blocks."""

        self.lib_directory = lib_directory
        self.localimport = localimport.localimport(
            sys.path + [str(self.lib_directory.absolute())], do_autodisable=False
        )

    def install(self, pip_args: list[str]) -> None:
        if not pip_args:
            raise ValueError("no pip_args specified")

        hash_file = self.lib_directory / "lib.hash"

        command = ["pip", "install"] + pip_args
        command += ["--target", str(self.lib_directory)]

        command_hash = hashlib.md5("!:!".join(command).encode()).hexdigest()
        if self.lib_directory.exists() and hash_file.exists() and hash_file.read_text().strip() == command_hash:
            logger.info("Skip bootstrapping environment as it is already initialized.")
            return

        hash_file.parent.mkdir(parents=True, exist_ok=True)
        hash_file.write_text(command_hash)

        sp.check_call([sys.executable, "-m"] + command)

    def activate(self) -> None:
        self.localimport.__enter__()

    def deactivate(self) -> None:
        self.localimport.__exit__()



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Any, Iterator, Optional

from .pyenv import PyenvManager

if TYPE_CHECKING:
    from .project import Project
    from .task import Task


class BuildContext:
    """This class is the single instance where all components of a build process come together."""

    def __init__(self, build_directory: Path) -> None:
        self._root_project: Optional[Project] = None
        self.build_directory = build_directory
        self.pyenv = PyenvManager(build_directory / ".lib")
        self.metadata: list[Any] = []

    def find_metadata(self, of_type: type[T]) -> T | None:
        """Returns the first entry in the :attr:`metadata` that is of the specified type."""

        return next((x for x in self.metadata if isinstance(x, of_type)), None)

    @property
    def root_project(self) -> Project:
        assert self._root_project is not None, "BuildContext.root_project is not set"
        return self._root_project

    @root_project.setter
    def root_project(self, project: Project) -> None:
        assert self._root_project is None, "BuildContext.root_project is already set"
        self._root_project = project

    def load_project(
        self,
        file: Path | None = None,
        directory: Path | None = None,
        parent: Project | None = None,
    ) -> Project:
        """Loads a project from a file or directory.

        Args:
            file: The file to load from. One of *file* or *directory* must be specified.
            directory: The directory to load from. If it is not specified, and the matching loader
                does not return a directory, the parent directory of *file* is used. If a directory
                is specified and the loader returns a different directory, the directory passed her
                takes precedence.
            parent: The parent project. If no parent is specified, then the :attr:`root_project`
                must not have been initialized yet and the loaded project will be initialize it.
                If the root project is initialized but no parent is specified, an error will be
                raised.
        """

        from .loader import detect_project_loader
        from .project import Project

        file, directory, loader = detect_project_loader(file, directory)

        project = Project(directory.name, directory, parent, self)

        if self._root_project is None:
            self._root_project = project

        loader.load_script(file, project)

        return project

    def iter_projects(self) -> Iterator[Project]:
        """Iterates over all projects in the context."""

        def _recurse(project: Project) -> Iterator[Project]:
            yield project
            for child_project in project.children().values():
                yield from _recurse(child_project)

        yield from _recurse(self.root_project)

    def resolve_tasks(self, targets: list[str] | None, relative_to: Project | None = None) -> list[Task]:
        """Resolve the given project or task references in *targets* relative to the specified project, or by
        default relative to the root project. A target is a colon-separated string that behaves similar to a
        filesystem path to address projects and tasks in the hierarchy. The root project is represented with a
        single colon and cannot be referenced by its name.

        A target that is just a task name will match all tasks of that name."""

        relative_to = relative_to or self.root_project

        if targets is None:
            # Return all default tasks.
            return [task for project in self.iter_projects() for task in project.tasks().values() if task.default]

        tasks: list[Task] = []
        count = 0
        target: str

        def _check_matched() -> None:
            nonlocal count
            if count == len(tasks):
                raise ValueError(f"no tasks matched selector {target!r}")
            count = len(tasks)

        for target in targets:
            count = len(tasks)

            if ":" not in target:
                # Select all targets with a name matching the specified target.
                tasks.extend(
                    task for project in self.iter_projects() for task in project.tasks().values() if task.name == target
                )
                _check_matched()
                continue

            # Resolve as many components in the project hierarchy as possible.
            project = relative_to
            parts = target.split(":")
            if parts[0] == "":
                project = self.root_project
                parts.pop(0)
            while parts:
                project_children = project.children()
                if parts[0] in project_children.values():
                    project = project_children[parts.pop(0)]
                else:
                    break

            project_tasks = project.tasks()
            if not parts or parts == [""]:
                # The project was selected, add all default tasks.
                tasks.extend(task for task in project_tasks.values() if task.default)
            elif len(parts) == 1:
                # A specific target is selected.
                if parts[0] not in project_tasks:
                    raise ValueError(f"task {target!r} does not exist")
                tasks.append(project_tasks[parts[0]])
            else:
                # Some project in the path does not exist.
                raise ValueError(f"project {':'.join(target.split(':')[:-1])} does not exist")

            _check_matched()

        return tasks

    def finalize(self) -> None:
        """Call :meth:`Task.finalize()` on all tasks. This should be called before a graph is created."""

        for project in self.iter_projects():
            for task in project.tasks().values():
                task.finalize()



""" Implements a loader for Python scripts. These Python scripts have access to importing certain build context
related members from the :mod:`kraken.api` module; note that these members are only available when imported in
the context of a script executed by this loader and not otherwise. """

from __future__ import annotations

import contextlib
import sys
import types
from pathlib import Path
from typing import Any, Iterator

from kraken.core.project import Project

from . import ProjectLoader

KRAKEN_DIR = Path(".kraken")
DEFAULT_FILES = [
    Path(".kraken.py"),
    Path("kraken.build.py"),
    Path(".kraken.build.py"),
    KRAKEN_DIR / "kraken.py",
    KRAKEN_DIR / "kraken.build.py",
    KRAKEN_DIR / "build.py",
]
REQUIREMENTS_FILE = KRAKEN_DIR / "requirements.txt"


class PythonScriptProjectLoader(ProjectLoader):
    def detect_in_project_directory(self, project_dir: Path) -> Path | None:
        for file in (project_dir / f for f in DEFAULT_FILES):
            if file.is_file():
                return file
        return None

    def match_file(self, file: Path) -> bool | Path | None:
        return file.suffix == ".py"

    def load_script(self, file: Path, project: Project) -> None:
        requirements_file = project.directory / REQUIREMENTS_FILE
        if requirements_file.is_file():
            pip_args = [x for x in requirements_file.read_text().splitlines() if not x.startswith("#") and x.strip()]
            if pip_args:
                # TODO (@NiklasRosenstein): Can we gather the requirements of included subprojects as well?
                # TODO (@NiklasRosenstein): We only want to do the install/activate once for the root project.
                project.context.pyenv.install(pip_args)
                project.context.pyenv.activate()

        with inject_kraken_api_module_for_project(project, file) as (api_module, project_module):
            code = compile(file.read_text(), filename=file, mode="exec")
            exec(code, vars(project_module))


@contextlib.contextmanager
def inject_kraken_api_module_for_project(
    project: Project,
    file: Path,
) -> Iterator[tuple[types.ModuleType, types.ModuleType]]:
    api_module_name = "kraken.api"

    # Create the temporary replacement for the kraken.api module that the script will import from.
    api_module: Any = types.ModuleType(api_module_name)
    api_module.ctx = project.context
    api_module.project = project

    # In order for @dataclass decorators to work in a Python script loaded by this build, it must be
    # able to look up the module in sys.modules.
    project_module = types.ModuleType(f"_kraken__{project.name}_{id(project)}")
    project_module.__file__ = str(file)

    old_module = sys.modules.get(api_module_name)
    try:
        sys.modules[api_module_name] = api_module
        sys.modules[project_module.__name__] = project_module
        yield api_module, project_module
    finally:
        del sys.modules[project_module.__name__]
        if old_module is None:
            sys.modules.pop(api_module_name)
        else:
            sys.modules[api_module_name] = old_module



from __future__ import annotations

import contextlib
import logging
import sys
from pathlib import Path
from typing import Iterator

import _pytest.fixtures
import pytest

from kraken.core.build_context import BuildContext
from kraken.core.build_graph import BuildGraph
from kraken.core.loader.python_script import inject_kraken_api_module_for_project
from kraken.core.project import Project
from kraken.core.task import Task, TaskResult

__version__ = "0.2.12"
logger = logging.getLogger(__name__)


def kraken_ctx() -> BuildContext:
    return BuildContext(Path(".build"))


@pytest.fixture(name="kraken_ctx")
def _kraken_ctx_fixture() -> BuildContext:
    return kraken_ctx()


@contextlib.contextmanager
def kraken_project(kraken_ctx: BuildContext, path: Path | None = None) -> Iterator[Project]:
    if path is None:
        path = Path(sys._getframe(1).f_code.co_filename)
    kraken_ctx.root_project = Project("test", path.parent, None, kraken_ctx)
    with inject_kraken_api_module_for_project(kraken_ctx.root_project, path) as (_api_module, _project_module):
        yield kraken_ctx.root_project


@pytest.fixture(name="kraken_project")
def _kraken_project_fixture(kraken_ctx: BuildContext, request: _pytest.fixtures.FixtureRequest) -> Iterator[Project]:
    with kraken_project(kraken_ctx, request.path) as project:
        yield project


def kraken_execute(ctx: BuildContext, targets: list[Task | str] | str) -> None:
    """A rudimentary but correct implementation for executing the tasks defined in the build context *ctx*. The
    task or tasks to be built must be specified explicitly with the *targets* parameter.

    !!! warning

        This function should only be used for testing. In production, you should rely on the :mod:`kraken.cli`
        package and its executor.
    """

    if isinstance(targets, str):
        targets = [targets]

    ctx.finalize()

    # Resolve string references to task objects.
    targets = targets or []
    tasks = ctx.resolve_tasks([t for t in targets if isinstance(t, str)]) + [
        t for t in targets if not isinstance(t, str)
    ]

    if not tasks:
        raise ValueError("no tasks selected")

    graph = BuildGraph(tasks)
    graph.trim()
    assert graph, "BuildGraph cannot be empty"

    for task in graph.execution_order():
        try:
            if task.is_skippable():
                logger.info("Skip task %s", task.path)
                continue
        except NotImplementedError:
            pass
        try:
            if task.is_up_to_date():
                logger.info("Task %s is up to date", task.path)
                continue
        except NotImplementedError:
            pass
        logger.info("Run task %s", task.path)
        result = task.execute()
        logger.info("Result of task %s is %r", task.path, result)
        if result not in (TaskResult.SUCCEEDED, TaskResult.SKIPPED, TaskResult.UP_TO_DATE):
            raise Exception(f"Task {task} result is {result.name}")



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Any, Iterable, Mapping, Optional, Type, TypeVar, cast

from .task import Task
from .utils import flatten

if TYPE_CHECKING:
    from .build_context import BuildContext

T = TypeVar("T")
T_Task = TypeVar("T_Task", bound="Task")


class Project:
    """A project consolidates tasks related to a directory on the filesystem."""

    name: str
    directory: Path
    parent: Optional[Project]
    context: BuildContext
    metadata: list[Any]  #: A list of arbitrary objects that are usually looked up by type.

    def __init__(self, name: str, directory: Path, parent: Optional[Project], context: BuildContext) -> None:
        self.name = name
        self.directory = directory
        self.parent = parent
        self.context = context
        self.metadata = []

        # We store all members that can be referenced by a fully qualified name in the same dictionary to ensure
        # we're not accidentally allocating the same name twice.
        self._members: dict[str, Task | Project] = {}

    def __repr__(self) -> str:
        return f"Project({self.path})"

    @property
    def path(self) -> str:
        """Returns the path that uniquely identifies the project in the current build context."""

        if self.parent is None:
            return ":"
        elif self.parent.parent is None:
            return f":{self.name}"
        else:
            return f"{self.parent.path}:{self.name}"

    @property
    def build_directory(self) -> Path:
        """Returns the recommended build directory for the project; this is a directory inside the context
        build directory ammended by the project name."""

        return self.context.build_directory / self.path.replace(":", "/").lstrip("/")

    def tasks(self) -> Mapping[str, Task]:
        return {t.name: t for t in self._members.values() if isinstance(t, Task)}

    def children(self) -> Mapping[str, Project]:
        return {p.name: p for p in self._members.values() if isinstance(p, Project)}

    def resolve_tasks(self, tasks: Iterable[str | Task]) -> list[Task]:
        return list(
            flatten(self.context.resolve_tasks([task], self) if isinstance(task, str) else [task] for task in tasks)
        )

    def add_task(self, task: Task) -> None:
        """Adds a task to the project.

        Raises:
            ValueError: If a member with the same name already exists or if the task's project does not match
        """

        if task.name in self._members:
            raise ValueError(f"{self} already has a member {task.name!r}, cannot add {task}")
        if task.project is not self:
            raise ValueError(f"{task}.project mismatch")
        self._members[task.name] = task

    def add_child(self, project: Project) -> None:
        """Adds a project as a child project.

        Raises:
            ValueError: If a member with the same name already exists or if the project's parent does not match
        """

        if project.name in self._members:
            raise ValueError(f"{self} already has a member {project.name!r}, cannot add {project}")
        if project.parent is not self:
            raise ValueError(f"{project}.parent mismatch")
        self._members[project.name] = project

    def do(
        self,
        name: str,
        task_type: Type[T_Task] = cast(Any, Task),
        default: bool = True,
        capture: bool = True,
    ) -> T_Task:
        """Add a task to the project under the given name, executing the specified action."""

        if name in self._members:
            raise ValueError(f"{self} already has a member {name!r}")

        task = task_type(name, self)
        task.default = default
        task.capture = capture
        self.add_task(task)
        return task

    def find_metadata(self, of_type: type[T]) -> T | None:
        """Returns the first entry in the :attr:`metadata` that is of the specified type."""

        return next((x for x in self.metadata if isinstance(x, of_type)), None)



from __future__ import annotations

import contextlib
import enum
import importlib
import os
import shutil
import tempfile
from pathlib import Path
from typing import IO, AnyStr, BinaryIO, ContextManager, Iterable, Iterator, TextIO, TypeVar, overload

from typing_extensions import Literal

T = TypeVar("T")


def flatten(it: Iterable[Iterable[T]]) -> Iterable[T]:
    for item in it:
        yield from item


def not_none(v: T | None) -> T:
    if v is None:
        raise RuntimeError("expected not-None")
    return v


class NotSet(enum.Enum):
    Value = 1


@overload
def atomic_file_swap(
    path: str | Path,
    mode: Literal["w"],
    always_revert: bool = ...,
    create_dirs: bool = ...,
) -> ContextManager[TextIO]:
    ...


@overload
def atomic_file_swap(
    path: str | Path,
    mode: Literal["wb"],
    always_revert: bool = ...,
    create_dirs: bool = ...,
) -> ContextManager[BinaryIO]:
    ...


@contextlib.contextmanager  # type: ignore
def atomic_file_swap(
    path: str | Path,
    mode: Literal["w", "wb"],
    always_revert: bool = False,
    create_dirs: bool = False,
) -> Iterator[IO[AnyStr]]:
    """Performs an atomic write to a file while temporarily moving the original file to a different random location.

    Args:
        path: The path to replace.
        mode: The open mode for the file (text or binary).
        always_revert: If enabled, swap the old file back into place even if the with context has no errors.
        create_dirs: If the file does not exist, and neither do its parent directories, create the directories.
            The directory will be removed if the operation is reverted.
    """

    path = Path(path)

    with contextlib.ExitStack() as exit_stack:
        if path.is_file():
            old = exit_stack.enter_context(
                tempfile.NamedTemporaryFile(
                    mode,
                    prefix=path.stem + "~",
                    suffix="~" + path.suffix,
                    dir=path.parent,
                )
            )
            old.close()
            os.rename(path, old.name)
        else:
            old = None

        def _revert() -> None:
            assert isinstance(path, Path)
            if path.is_file():
                path.unlink()
            if old is not None:
                os.rename(old.name, path)

        if not path.parent.is_dir() and create_dirs:
            path.parent.mkdir(exist_ok=True)
            _old_revert = _revert

            def _revert() -> None:
                assert isinstance(path, Path)
                try:
                    shutil.rmtree(path.parent)
                finally:
                    _old_revert()

        try:
            with path.open(mode) as new:
                yield new
        except BaseException:
            _revert()
            raise
        else:
            if always_revert:
                _revert()
            else:
                if old is not None:
                    os.remove(old.name)


@overload
def import_class(fqn: str) -> type:
    ...


@overload
def import_class(fqn: str, base_type: type[T]) -> type[T]:
    ...


def import_class(fqn: str, base_type: type[T] | None = None) -> type[T]:
    mod_name, cls_name = fqn.rpartition(".")[::2]
    module = importlib.import_module(mod_name)
    cls = getattr(module, cls_name)
    if not isinstance(cls, type):
        raise TypeError(f"expected type object at {fqn!r}, got {type(cls).__name__}")
    if base_type is not None and not issubclass(cls, base_type):
        raise TypeError(f"expected subclass of {base_type} at {fqn!r}, got {cls}")
    return cls



from __future__ import annotations

import abc
import logging
from pathlib import Path
from typing import Callable, Iterable

import pkg_resources

from kraken.core.project import Project

ENTRYPOINT = "kraken.core.loader"

logger = logging.getLogger(__name__)


class ProjectLoaderError(Exception):
    pass


class ProjectLoader(abc.ABC):
    @abc.abstractmethod
    def detect_in_project_directory(self, project_dir: Path) -> Path | None:
        """
        Args:
            project_dir (Path): The directory to search for a build script that can be loaded.
        Returns:
            Path: The path to the file that should be loaded.
            None: If the directory does not contain a file that can be loaded.
        """

    @abc.abstractmethod
    def match_file(self, file: Path) -> bool | Path | None:
        """
        Args:
            file (Path): A file to check if it can be loaded by this loader.
        Returns:
            bool: `True` if the file can be loaded, `False` if not.
            Path: The path to the project directory belonging to the file, if the file can be loaded.
            None: If the file cannot be loaded.
        """

    @abc.abstractmethod
    def load_script(self, file: Path, project: Project) -> None:
        """
        Args:
            file (Path): The build script to load.
            project (Project): The project to load the build script into.
        """


def get_loader_implementations() -> Iterable[ProjectLoader]:
    """Iterate over the registered loader implementations."""

    for ep in pkg_resources.iter_entry_points(ENTRYPOINT):
        try:
            factory: Callable[[], ProjectLoader] = ep.load()
            yield factory()
        except Exception:
            logger.exception("an unhandled exception occurred while fetching BuildScriptLoader implementation: %s", ep)


def detect_project_loader(file: Path | None, directory: Path | None) -> tuple[Path, Path, ProjectLoader]:
    """Detects the loader for the given *file* or *directory*. Both may be specified, but at least one must
    present. If only a directory is given, the loader must report the file to load. If only a file is given,
    the loader _may_ report the respective project directory.

    Raises:
        ProjectLoaderError: If the given file and/or directory combination cannot be loaded by any loader.
    """

    if file is None:
        if directory is None:
            raise ValueError("need file or directory")
        for loader in get_loader_implementations():
            file = loader.detect_in_project_directory(directory)
            if file:
                break
        else:
            raise ProjectLoaderError(f'"{directory}" does not look like a Kraken project directory')
    else:
        for loader in get_loader_implementations():
            match = loader.match_file(file)
            if isinstance(match, Path):
                if directory is None:
                    directory = match
                break
            elif match:
                if directory is None:
                    directory = file.parent
                break
        else:
            raise ProjectLoaderError(f'"{file}" is not accepted by any project loader')

    return file, directory, loader



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src',
 'core': 'src/core',
 'core.loader': 'src/core/loader',
 'testing': 'src/testing'}

packages = \
['api', 'core', 'core.loader', 'testing']

package_data = \
{'': ['*']}

install_requires = \
['localimport>=1.7.6,<2.0.0',
 'networkx>=2.6.0,<3.0.0',
 'setuptools>=33.1.0',
 'typeapi>=0.2.1,<0.3.0']

extras_require = \
{'testing': ['pytest>=6.0.0']}

entry_points = \
{'kraken.core.loader': ['python_script = '
                        'kraken.core.loader.python_script:PythonScriptProjectLoader'],
 'pytest11': ['kraken-testing = kraken.testing']}

setup_kwargs = {
    'name': 'kraken-core',
    'version': '0.2.12',
    'description': '',
    'long_description': '# kraken-core\n\n[![Python application](https://github.com/kraken-build/kraken-core/actions/workflows/python-package.yml/badge.svg)](https://github.com/kraken-build/kraken-core/actions/workflows/python-package.yml)\n[![PyPI version](https://badge.fury.io/py/kraken-core.svg)](https://badge.fury.io/py/kraken-core)\n\nThe `kraken.core` package provides the primitives of describing a build and deriving build tasks.\n\nAside from the `kraken.core` package, this package also provides the `kraken.api` module that is\nused only at runtime by Kraken build scripts and the `kraken.testing` module for Pytest fixtures.\n\n## How does it work?\n\nKraken uses **tasks** to describe units of work that can be chained and establish dependencies between each other.\nEach task has a **schema** that defines its input and output properties. When an output property is linked to the\ninput property of another task, this established as dependency between the tasks.\n\n```py\nfrom kraken.std.docker_gen import generate_dockerfile\nfrom kraken.std.docker_build import build_docker_image\ndockerfile = generate_dockerfile(source_file="Dockerfile.yml")\nbuild_docker_image(dockerfile=dockerfile.path, tags=["example:latest"], load=True)\n```\n\nThis populates the project with two **tasks** and connects the computed output property of one to the other,\nallowing the tasks that will run for `build_docker_image()` to pick up the dynamically generated Dockerfile that\nis written into a location in the build directory by the `generate_dockerfile()` task.\n\n<p align="center"><img src="assets/graph.png" height="225px"></p>\n\n## Core API\n\nKraken **tasks** are described with a schema. Each schema field has a type and may be an input or output parameter.\nOutput parameters are only available once a resource is executed; Kraken will that a proper execution order is\nestablished such that output properties are hydrated before another resource tries to access them as an input.\n\n```py\nfrom kraken.core.task import Context, Task, Property, Output, task_factory\nfrom typing_extensions import Annotated\n\nclass GenerateDockerfileTask(Task):\n    source_file: Property[str]\n    path: Annotated[Property[str], Output]\n\n    def execute(self, ctx: Context) -> None:\n        path = Path(self.path.setdefault(str(ctx.build_directory / "Dockerfile")))\n        path.write_text(render_dockerfile(Path(self.source_file.get()).read_text()))\n\ngenerate_dockerfile = task_factory(GenerateDockerfileTask)\n```\n\n### Notes on writing extensions\n\n#### Task properties\n\nThe Kraken code base uses the 3.10+ type union operator `|` for type hints where possible. However, special care needs\nto be taken with this operator when defining properties on Kraken tasks. The annotations on task objects are eveluated\nand will cause errors in Python versions lower than 3.10 when using the union operator `|` even with\n`__future__.annotations` enabled.\n\n<table><tr><th>Do</th><th>Don\'t</th></tr>\n<tr><td>\n\n```py\nfrom __future__ import annotations\nfrom typing import Union\nfrom kraken.core.property import Property\nfrom kraken.core.task import Task\n\n\nclass MyTask(Task):\n    my_prop: Property[Union[str, Path]]\n\n    def _internal_method(self, value: str | Path) -> None:\n        ...\n```\n\n</td><td>\n\n\n```py\nfrom __future__ import annotations\nfrom typing import Union\nfrom kraken.core.property import Property\nfrom kraken.core.task import Task\n\n\nclass MyTask(Task):\n    my_prop: Property[str | Path]  # unsupported operand type(s) for |: \'type\' and \'type\'\n\n    def _internal_method(self, value: str | Path) -> None:\n        ...\n```\n\n</td></tr>\n</table>\n\nAlso note that properties use "value adapters" to validate and coerce values to the property value type. Depending on\nthe order of union types, this may change the semantics of the value stored in a property. For example, the value\nadapter for the `pathlib.Path` type will convert strings to a path object. If your property accepts both of these\ntypes, putting the `str` type first in the union will ensure that your property keeps the string a string instead of\ncoercing it to a path.\n\n## Integration testing API\n\nThe `kraken.testing` module provides Pytest fixtures for integration testing Kraken extension modules. The\n`kraken_project` fixture provides you with access to a Kraken project object. The `kraken.testing.execute()`\nfunction is a rudimentary implementation to correctly execute a build graph, but it is not recommended for\nproduction use and should be used in tests only.\n\n__Example__\n\n```py\ndef test__helm_push_to_oci_registry(kraken_project: Project, oci_registry: str) -> None:\n    """This integration test publishes a Helm chart to a local registry and checks if after publishing it, the\n    chart can be accessed via the registry."""\n\n    helm_settings(kraken_project).add_auth(oci_registry, USER_NAME, USER_PASS, insecure=True)\n    package = helm_package(chart_directory="data/example-chart")\n    helm_push(chart_tarball=package.chart_tarball, registry=f"oci://{oci_registry}/example")\n    kraken_project.context.execute([":helmPush"])\n    response = httpx.get(f"http://{oci_registry}/v2/example/example-chart/tags/list", auth=(USER_NAME, USER_PASS))\n    response.raise_for_status()\n    tags = response.json()\n    assert tags == {"name": "example/example-chart", "tags": ["0.1.0"]}\n```\n\n> This is a working example from the `kraken-std` package.\n',
    'author': 'Niklas Rosenstein',
    'author_email': 'rosensteinniklas@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
